@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -15,6 +15,7 @@
 
 #include "collect.h"
 #include "dnd.h"
+#include "editors.h"
 #include "filelist.h"
 #include "fullscreen.h"
 #include "image.h"
@@ -22,6 +23,7 @@
 #include "menu.h"
 #include "slideshow.h"
 #include "utilops.h"
+#include "ui_bookmark.h"
 #include "ui_fileops.h"
 #include "ui_menu.h"
 
@@ -39,6 +41,7 @@ struct _ViewWindow
 	SlideShowData *ss;
 
 	GList *list;
+	GList *list_pointer;
 };
 
 
@@ -71,30 +74,12 @@ static ImageWindow *view_window_active_image(ViewWindow *vw)
 	return vw->imd;
 }
 
-static GList *path_list_copy(GList *list)
-{
-	GList *new_list = NULL;
-	GList *work;
-
-	work = list;
-	while (work)
-		{
-		gchar *path;
-
-		path = work->data;
-		work = work->next;
-
-		new_list = g_list_prepend(new_list, g_strdup(path));
-		}
-
-	return g_list_reverse(new_list);
-}
-
 static void view_window_set_list(ViewWindow *vw, GList *list)
 {
 
 	path_list_free(vw->list);
 	vw->list = NULL;
+	vw->list_pointer = NULL;
 
 	vw->list = path_list_copy(list);
 }
@@ -175,7 +160,6 @@ static void view_collection_step_to_end(ViewWindow *vw, gint last)
 		image_change_from_collection(imd, cd, info, image_zoom_get_default(imd, zoom_mode));
 		if (read_ahead_info) image_prebuffer_set(imd, read_ahead_info->path);
 		}
-	
 }
 
 static void view_list_step(ViewWindow *vw, gint next)
@@ -184,28 +168,35 @@ static void view_list_step(ViewWindow *vw, gint next)
 	const gchar *path;
 	GList *work;
 	GList *work_ahead;
-	gint found;
 
 	if (!vw->list) return;
 
 	path = image_get_path(imd);
 	if (!path) return;
 
-	found = FALSE;
-	work = vw->list;
-	while (work && !found)
+	if (g_list_position(vw->list, vw->list_pointer) >= 0)
 		{
-		gchar *temp;
+		work = vw->list_pointer;
+		}
+	else
+		{
+		gint found = FALSE;
 
-		temp = work->data;
+		work = vw->list;
+		while (work && !found)
+			{
+			gchar *temp;
 
-		if (strcmp(path, temp) == 0)
-			{
-			found = TRUE;
-			}
-		else
-			{
-			work = work->next;
+			temp = work->data;
+
+			if (strcmp(path, temp) == 0)
+				{
+				found = TRUE;
+				}
+			else
+				{
+				work = work->next;
+				}
 			}
 		}
 	if (!work) return;
@@ -225,6 +216,7 @@ static void view_list_step(ViewWindow *vw, gint next)
 
 	path = work->data;
 	image_change_path(imd, path, image_zoom_get_default(imd, zoom_mode));
+	vw->list_pointer = work;
 
 	if (enable_read_ahead && work_ahead)
 		{
@@ -255,6 +247,7 @@ static void view_list_step_to_end(ViewWindow *vw, gint last)
 
 	path = work->data;
 	image_change_path(imd, path, image_zoom_get_default(imd, zoom_mode));
+	vw->list_pointer = work;
 
 	if (enable_read_ahead && work_ahead)
 		{
@@ -313,11 +306,22 @@ static void view_step_to_end(ViewWindow *vw, gint last)
  *-----------------------------------------------------------------------------
  */
 
+static void view_window_menu_pos_cb(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer data)
+{
+	ViewWindow *vw = data;
+	ImageWindow *imd;
+
+	imd = view_window_active_image(vw);
+	gdk_window_get_origin(imd->image->window, x, y);
+	popup_menu_position_clamp(menu, x, y, 0);
+}
+
 static gint view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
 	ViewWindow *vw = data;
 	ImageWindow *imd;
 	gint stop_signal = FALSE;
+	GtkWidget *menu;
 	gint x = 0;
 	gint y = 0;
 
@@ -350,6 +354,7 @@ static gint view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpoi
 			break;
 		case GDK_Page_Down: case GDK_KP_Page_Down:
 		case GDK_space:
+		case 'N': case 'n':
 			view_step_next(vw);
 			stop_signal = TRUE;
 			break;
@@ -401,6 +406,7 @@ static gint view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpoi
 		case 'P': case 'p':
 			slideshow_pause_toggle(vw->ss);
 			break;
+		case 'F': case 'f':
 		case 'V': case 'v':
 			view_fullscreen_toggle(vw, FALSE);
 			stop_signal = TRUE;
@@ -432,6 +438,12 @@ static gint view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpoi
 				}
 			stop_signal = TRUE;
 			break;
+		case GDK_Menu:
+		case GDK_F10:
+			menu = view_popup_menu(vw);
+			gtk_menu_popup(GTK_MENU(menu), NULL, NULL, view_window_menu_pos_cb, vw, 0, GDK_CURRENT_TIME);
+			stop_signal = TRUE;
+			break;
 		}
 
 	if (event->state & GDK_CONTROL_MASK)
@@ -463,6 +475,12 @@ static gint view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpoi
 			case '8':
 				n = 7;
 				break;
+			case '9':
+				n = 8;
+				break;
+			case '0':
+				n = 9;
+				break;
 			case 'C': case 'c':
 				view_fullscreen_toggle(vw, TRUE);
 				file_util_copy(image_get_path(vw->imd), NULL, NULL);
@@ -483,6 +501,11 @@ static gint view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpoi
 				file_util_delete(image_get_path(vw->imd), NULL);
 				stop_signal = TRUE;
 				break;
+			case 'P': case 'p':
+				view_fullscreen_toggle(vw, TRUE);
+				info_window_new(image_get_path(vw->imd), NULL);
+				stop_signal = TRUE;
+				break;
 			case 'W': case 'w':
 				view_window_close(vw);
 				break;
@@ -531,39 +554,38 @@ static gint view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpoi
  *-----------------------------------------------------------------------------
  */ 
 
-static void button1_cb(ImageWindow *imd, GdkEventButton *bevent, gpointer data)
+static void button1_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
 {
 	ViewWindow *vw = data;
 
 	view_step_next(vw);
 }
 
-static void button2_cb(ImageWindow *imd, GdkEventButton *bevent, gpointer data)
+static void button2_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
 {
 	ViewWindow *vw = data;
 
 	view_step_prev(vw);
 }
 
-static void button3_cb(ImageWindow *imd, GdkEventButton *bevent, gpointer data)
+static void button3_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
 {
 	ViewWindow *vw = data;
 	GtkWidget *menu;
 
 	menu = view_popup_menu(vw);
-	gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL,
-		bevent->button, bevent->time);
+	gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL, 3, time);
 }
 
-static void button4_cb(ImageWindow *imd, GdkEventButton *bevent, gpointer data)
+static void button4_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
 {
 	ViewWindow *vw = data;
 
-	if (bevent->state & GDK_CONTROL_MASK)
+	if (state & GDK_CONTROL_MASK)
 		{
-		image_zoom_adjust_at_point(imd, get_zoom_increment(), bevent->x, bevent->y);
+		image_zoom_adjust_at_point(imd, get_zoom_increment(), x, y);
 		}
-	else if ( (bevent->state & GDK_SHIFT_MASK) != (mousewheel_scrolls))
+	else if ( (state & GDK_SHIFT_MASK) != (mousewheel_scrolls))
 		{
 		image_scroll(imd, 0, -MOUSEWHEEL_SCROLL_SIZE);
 		}
@@ -573,15 +595,15 @@ static void button4_cb(ImageWindow *imd, GdkEventButton *bevent, gpointer data)
 		}
 }
 
-static void button5_cb(ImageWindow *imd, GdkEventButton *bevent, gpointer data)
+static void button5_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
 {
 	ViewWindow *vw = data;
 
-	if (bevent->state & GDK_CONTROL_MASK)
+	if (state & GDK_CONTROL_MASK)
 		{
-		image_zoom_adjust_at_point(imd, -get_zoom_increment(), bevent->x, bevent->y);
+		image_zoom_adjust_at_point(imd, -get_zoom_increment(), x, y);
 		}
-	else if ( (bevent->state & GDK_SHIFT_MASK) != (mousewheel_scrolls))
+	else if ( (state & GDK_SHIFT_MASK) != (mousewheel_scrolls))
 		{
 		image_scroll(imd, 0, MOUSEWHEEL_SCROLL_SIZE);
 		}
@@ -623,8 +645,8 @@ static void view_fullscreen_toggle(ViewWindow *vw, gint force_off)
 		vw->fs = fullscreen_start(vw->window, vw->imd, view_fullscreen_stop_func, vw);
 
 		view_image_set_buttons(vw, vw->fs->imd);
-		gtk_signal_connect(GTK_OBJECT(vw->fs->window), "key_press_event",
-				   GTK_SIGNAL_FUNC(view_window_key_press_cb), vw);
+		g_signal_connect(G_OBJECT(vw->fs->window), "key_press_event",
+				 G_CALLBACK(view_window_key_press_cb), vw);
 
 		if (vw->ss) vw->ss->imd = vw->fs->imd;
 		}
@@ -659,6 +681,7 @@ static void view_slideshow_start(ViewWindow *vw)
 			vw->ss = slideshow_start_from_path_list(view_window_active_image(vw),
 								path_list_copy(vw->list),
 								view_slideshow_stop_func, vw);
+			vw->list_pointer = NULL;
 			return;
 			}
 
@@ -699,6 +722,7 @@ static void real_view_window_new(const gchar *path, GList *list, CollectionData
 {
 	ViewWindow *vw;
 	GtkAllocation req_size;
+	GdkGeometry geometry;
 	gint w, h;
 
 	if (!path && !list && (!cd || !info)) return;
@@ -707,12 +731,18 @@ static void real_view_window_new(const gchar *path, GList *list, CollectionData
 	vw->fs = NULL;
 	vw->ss = NULL;
 	vw->list = NULL;
+	vw->list_pointer = NULL;
 
 	vw->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_policy(GTK_WINDOW(vw->window), TRUE, TRUE, FALSE);
-	gtk_window_set_title (GTK_WINDOW (vw->window), "GQview");
-        gtk_window_set_wmclass(GTK_WINDOW (vw->window), "view", "GQview");
-        gtk_container_border_width (GTK_CONTAINER (vw->window), 0);
+
+	geometry.min_width = 8;
+	geometry.min_height = 8;
+	gtk_window_set_geometry_hints(GTK_WINDOW(vw->window), NULL, &geometry, GDK_HINT_MIN_SIZE);
+
+	gtk_window_set_resizable(GTK_WINDOW(vw->window), TRUE);
+	gtk_window_set_title (GTK_WINDOW(vw->window), "GQview");
+        gtk_window_set_wmclass(GTK_WINDOW(vw->window), "view", "GQview");
+        gtk_container_set_border_width(GTK_CONTAINER(vw->window), 0);
 
 	window_set_icon(vw->window, (const gchar **)view_xpm, NULL);
 
@@ -720,8 +750,9 @@ static void real_view_window_new(const gchar *path, GList *list, CollectionData
 
 	if (black_window_background) image_background_set_black(vw->imd, TRUE);
 
-	image_attach_window(vw->imd, vw->window, NULL, " - GQview", TRUE);
+	image_attach_window(vw->imd, vw->window, NULL, "GQview", TRUE);
 
+	image_auto_refresh(vw->imd, 0);
 	image_top_window_set_sync(vw->imd, TRUE);
 
 	gtk_container_add(GTK_CONTAINER(vw->window), vw->imd->widget);
@@ -731,10 +762,10 @@ static void real_view_window_new(const gchar *path, GList *list, CollectionData
 
 	view_image_set_buttons(vw, vw->imd);
 
-	gtk_signal_connect(GTK_OBJECT(vw->window), "delete_event",
-			   GTK_SIGNAL_FUNC(view_window_delete_cb), vw);
-	gtk_signal_connect(GTK_OBJECT(vw->window), "key_press_event",
-			   GTK_SIGNAL_FUNC(view_window_key_press_cb), vw);
+	g_signal_connect(G_OBJECT(vw->window), "delete_event",
+			 G_CALLBACK(view_window_delete_cb), vw);
+	g_signal_connect(G_OBJECT(vw->window), "key_press_event",
+			 G_CALLBACK(view_window_key_press_cb), vw);
 	if (cd && info)
 		{
 		image_change_from_collection(vw->imd, cd, info, image_zoom_get_default(NULL, zoom_mode));
@@ -749,6 +780,8 @@ static void real_view_window_new(const gchar *path, GList *list, CollectionData
 		{
 		view_window_set_list(vw, list);
 		image_change_path(vw->imd, (gchar *)vw->list->data, image_zoom_get_default(NULL, zoom_mode));
+		vw->list_pointer = vw->list;
+
 		if (enable_read_ahead)
 			{
 			GList *work = vw->list->next;
@@ -779,13 +812,13 @@ static void real_view_window_new(const gchar *path, GList *list, CollectionData
 		if (h > mh) h = mh;
 		}
 
-	gtk_window_set_default_size (GTK_WINDOW(vw->window), w, h);
+	gtk_window_set_default_size(GTK_WINDOW(vw->window), w, h);
 	req_size.x = req_size.y = 0;
 	req_size.width = w;
 	req_size.height = h;
 	gtk_widget_size_allocate(GTK_WIDGET(vw->window), &req_size);
 
-	gtk_widget_set_usize(vw->imd->eventbox, w, h);
+	gtk_widget_set_size_request(vw->imd->eventbox, w, h);
 
 	gtk_widget_show(vw->window);
 
@@ -880,6 +913,7 @@ static void view_info_cb(GtkWidget *widget, gpointer data)
 {
 	ViewWindow *vw = data;
 
+	view_fullscreen_toggle(vw, TRUE);
 	info_window_new(image_get_path(vw->imd), NULL);
 }
 
@@ -995,63 +1029,63 @@ static GtkWidget *view_popup_menu(ViewWindow *vw)
 
 	menu = popup_menu_short_lived();
 
-	menu_item_add(menu, _("Zoom in"), GTK_SIGNAL_FUNC(view_zoom_in_cb), vw);
-	menu_item_add(menu, _("Zoom out"), GTK_SIGNAL_FUNC(view_zoom_out_cb), vw);
-	menu_item_add(menu, _("Zoom 1:1"), GTK_SIGNAL_FUNC(view_zoom_1_1_cb), vw);
-	menu_item_add(menu, _("Fit image to window"), GTK_SIGNAL_FUNC(view_zoom_fit_cb), vw);
+	menu_item_add_stock(menu, _("Zoom _in"), GTK_STOCK_ZOOM_IN, G_CALLBACK(view_zoom_in_cb), vw);
+	menu_item_add_stock(menu, _("Zoom _out"), GTK_STOCK_ZOOM_OUT, G_CALLBACK(view_zoom_out_cb), vw);
+	menu_item_add_stock(menu, _("Zoom _1:1"), GTK_STOCK_ZOOM_100, G_CALLBACK(view_zoom_1_1_cb), vw);
+	menu_item_add_stock(menu, _("Fit image to _window"), GTK_STOCK_ZOOM_FIT, G_CALLBACK(view_zoom_fit_cb), vw);
 	menu_item_add_divider(menu);
 
-	item = submenu_add_edit(menu, NULL, GTK_SIGNAL_FUNC(view_edit_cb), vw);
+	item = submenu_add_edit(menu, NULL, G_CALLBACK(view_edit_cb), vw);
 	menu_item_add_divider(item);
-	menu_item_add(item, _("Set as wallpaper"), GTK_SIGNAL_FUNC(view_wallpaper_cb), vw);
+	menu_item_add(item, _("Set as _wallpaper"), G_CALLBACK(view_wallpaper_cb), vw);
 
-	submenu_add_alter(menu, GTK_SIGNAL_FUNC(view_alter_cb), vw);
+	submenu_add_alter(menu, G_CALLBACK(view_alter_cb), vw);
 
-	menu_item_add(menu, _("Properties"), GTK_SIGNAL_FUNC(view_info_cb), vw);
+	menu_item_add_stock(menu, _("_Properties"), GTK_STOCK_PROPERTIES, G_CALLBACK(view_info_cb), vw);
 
-	menu_item_add(menu, _("View in new window"), GTK_SIGNAL_FUNC(view_new_window_cb), vw);
+	menu_item_add_stock(menu, _("View in _new window"), GTK_STOCK_NEW, G_CALLBACK(view_new_window_cb), vw);
 
 	menu_item_add_divider(menu);
-	menu_item_add(menu, _("Copy..."), GTK_SIGNAL_FUNC(view_copy_cb), vw);
-	menu_item_add(menu, _("Move..."), GTK_SIGNAL_FUNC(view_move_cb), vw);
-	menu_item_add(menu, _("Rename..."), GTK_SIGNAL_FUNC(view_rename_cb), vw);
-	menu_item_add(menu, _("Delete..."), GTK_SIGNAL_FUNC(view_delete_cb), vw);
+	menu_item_add_stock(menu, _("_Copy..."), GTK_STOCK_COPY, G_CALLBACK(view_copy_cb), vw);
+	menu_item_add(menu, _("_Move..."), G_CALLBACK(view_move_cb), vw);
+	menu_item_add(menu, _("_Rename..."), G_CALLBACK(view_rename_cb), vw);
+	menu_item_add_stock(menu, _("_Delete..."), GTK_STOCK_DELETE, G_CALLBACK(view_delete_cb), vw);
 
 	menu_item_add_divider(menu);
 
 	if (vw->ss)
 		{
-		menu_item_add(menu, _("Stop slideshow"), GTK_SIGNAL_FUNC(view_slideshow_stop_cb), vw);
+		menu_item_add(menu, _("_Stop slideshow"), G_CALLBACK(view_slideshow_stop_cb), vw);
 		if (slideshow_paused(vw->ss))
 			{
-			item = menu_item_add(menu, _("Continue slideshow"),
-					     GTK_SIGNAL_FUNC(view_slideshow_pause_cb), vw);
+			item = menu_item_add(menu, _("Continue slides_how"),
+					     G_CALLBACK(view_slideshow_pause_cb), vw);
 			}
 		else
 			{
-			item = menu_item_add(menu, _("Pause slideshow"),
-					     GTK_SIGNAL_FUNC(view_slideshow_pause_cb), vw);
+			item = menu_item_add(menu, _("Pause slides_how"),
+					     G_CALLBACK(view_slideshow_pause_cb), vw);
 			}
 		}
 	else
 		{
-		item = menu_item_add(menu, _("Start slideshow"), GTK_SIGNAL_FUNC(view_slideshow_start_cb), vw);
+		item = menu_item_add(menu, _("_Start slideshow"), G_CALLBACK(view_slideshow_start_cb), vw);
 		gtk_widget_set_sensitive(item, (vw->list != NULL) || view_window_contains_collection(vw));
-		item = menu_item_add(menu, _("Pause slideshow"), GTK_SIGNAL_FUNC(view_slideshow_pause_cb), vw);
+		item = menu_item_add(menu, _("Pause slides_how"), G_CALLBACK(view_slideshow_pause_cb), vw);
 		gtk_widget_set_sensitive(item, FALSE);
 		}
 
 	if (vw->fs)
 		{
-		menu_item_add(menu, _("Exit full screen"), GTK_SIGNAL_FUNC(view_fullscreen_cb), vw);
+		menu_item_add(menu, _("Exit _full screen"), G_CALLBACK(view_fullscreen_cb), vw);
 		}
 	else
 		{
-		menu_item_add(menu, _("Full screen"), GTK_SIGNAL_FUNC(view_fullscreen_cb), vw);
+		menu_item_add(menu, _("_Full screen"), G_CALLBACK(view_fullscreen_cb), vw);
 		}
 
 	menu_item_add_divider(menu);
-	menu_item_add(menu, _("Close window"), GTK_SIGNAL_FUNC(view_close_cb), vw);
+	menu_item_add_stock(menu, _("C_lose window"), GTK_STOCK_CLOSE, G_CALLBACK(view_close_cb), vw);
 
 	return menu;
 }
@@ -1116,6 +1150,7 @@ static void view_dir_list_do(ViewWindow *vw, GList *list, gint skip, gint recurs
 
 		path = vw->list->data;
 		image_change_path(vw->imd, path, image_zoom_get_default(vw->imd, zoom_mode));
+		vw->list_pointer = vw->list;
 
 		work = vw->list->next;
 		if (enable_read_ahead && work)
@@ -1165,16 +1200,16 @@ static GtkWidget *view_confirm_dir_list(ViewWindow *vw, GList *list)
 	d->list = list;
 
 	menu = popup_menu_short_lived();
-	gtk_signal_connect(GTK_OBJECT(menu), "destroy",
-			   GTK_SIGNAL_FUNC(view_dir_list_destroy), d);
+	g_signal_connect(G_OBJECT(menu), "destroy",
+			 G_CALLBACK(view_dir_list_destroy), d);
 
-	menu_item_add(menu, _("Dropped list includes directories."), NULL, NULL);
+	menu_item_add_stock(menu, _("Dropped list includes directories."), GTK_STOCK_DND_MULTIPLE, NULL, NULL);
 	menu_item_add_divider(menu);
-	menu_item_add(menu, _("Add contents"), GTK_SIGNAL_FUNC(view_dir_list_add), d);
-	menu_item_add(menu, _("Add contents recursive"), GTK_SIGNAL_FUNC(view_dir_list_recurse), d);
-	menu_item_add(menu, _("Skip directories"), GTK_SIGNAL_FUNC(view_dir_list_skip), d);
+	menu_item_add_stock(menu, _("Add contents"), GTK_STOCK_OK, G_CALLBACK(view_dir_list_add), d);
+	menu_item_add_stock(menu, _("Add contents recursive"), GTK_STOCK_ADD, G_CALLBACK(view_dir_list_recurse), d);
+	menu_item_add_stock(menu, _("Skip directories"), GTK_STOCK_REMOVE, G_CALLBACK(view_dir_list_skip), d);
 	menu_item_add_divider(menu);
-	menu_item_add(menu, _("Cancel"), GTK_SIGNAL_FUNC(view_dir_list_cancel), d);
+	menu_item_add_stock(menu, _("Cancel"), GTK_STOCK_CANCEL, G_CALLBACK(view_dir_list_cancel), d);
 
 	return menu;
 }
@@ -1207,7 +1242,7 @@ static void view_window_get_dnd_data(GtkWidget *widget, GdkDragContext *context,
 			{
 			GList *work;
 
-			list = get_uri_file_list(selection_data->data);
+			list = uri_list_from_text(selection_data->data, TRUE);
 
 			work = list;
 			while (work)
@@ -1241,6 +1276,7 @@ static void view_window_get_dnd_data(GtkWidget *widget, GdkDragContext *context,
 				{
 				view_slideshow_stop(vw);
 				view_window_set_list(vw, NULL);
+
 				if (source && info_list)
 					{
 					image_change_from_collection(imd, source, info_list->data, image_zoom_get_default(imd, zoom_mode));
@@ -1251,6 +1287,8 @@ static void view_window_get_dnd_data(GtkWidget *widget, GdkDragContext *context,
 						{
 						vw->list = list;
 						list = NULL;
+
+						vw->list_pointer = vw->list;
 						}
 					image_change_path(imd, path, image_zoom_get_default(imd, zoom_mode));
 					}
@@ -1273,19 +1311,27 @@ static void view_window_set_dnd_data(GtkWidget *widget, GdkDragContext *context,
 	if (path)
 		{
 		gchar *text = NULL;
+		gint len;
+		gint plain_text;
+		GList *list;
+
 		switch (info)
 			{
 			case TARGET_URI_LIST:
-				text = g_strconcat("file:", path, "\r\n", NULL);
+				plain_text = FALSE;
 				break;
 			case TARGET_TEXT_PLAIN:
-				text = g_strdup(path);
+			default:
+				plain_text = TRUE;
 				break;
 			}
+		list = g_list_append(NULL, (gchar *)path);
+		text = uri_text_from_list(list, &len, plain_text);
+		g_list_free(list);
 		if (text)
 			{
 			gtk_selection_data_set (selection_data, selection_data->target,
-						8, text, strlen(text));
+						8, text, len);
 			g_free(text);
 			}
 		}
@@ -1305,15 +1351,15 @@ static void view_window_dnd_init(ViewWindow *vw)
 	gtk_drag_source_set(imd->image, GDK_BUTTON2_MASK,
 			    dnd_file_drag_types, dnd_file_drag_types_count,
 			    GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
-	gtk_signal_connect(GTK_OBJECT(imd->image), "drag_data_get",
-			   GTK_SIGNAL_FUNC(view_window_set_dnd_data), vw);
+	g_signal_connect(G_OBJECT(imd->image), "drag_data_get",
+			 G_CALLBACK(view_window_set_dnd_data), vw);
 
 	gtk_drag_dest_set(imd->image,
 			  GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
 			  dnd_file_drop_types, dnd_file_drop_types_count,
-                          GDK_ACTION_COPY);
-	gtk_signal_connect(GTK_OBJECT(imd->image), "drag_data_received",
-			   GTK_SIGNAL_FUNC(view_window_get_dnd_data), vw);
+                          GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
+	g_signal_connect(G_OBJECT(imd->image), "drag_data_received",
+			 G_CALLBACK(view_window_get_dnd_data), vw);
 }
 
 /*
@@ -1357,20 +1403,47 @@ static void view_real_removed(ViewWindow *vw, const gchar *path, GList *ignore_l
 	if (vw->list)
 		{
 		GList *work;
+		GList *old;
+
+		old = vw->list_pointer;
+
 		work = vw->list;
 		while (work)
 			{
 			gchar *chk_path;
+			GList *chk_link;
 
 			chk_path = work->data;
+			chk_link = work;
 			work = work->next;
 
 			if (strcmp(chk_path, path) == 0)
 				{
+				if (vw->list_pointer == chk_link)
+					{
+					vw->list_pointer = (chk_link->next) ? chk_link->next : chk_link->prev;
+					}
 				vw->list = g_list_remove(vw->list, chk_path);
 				g_free(chk_path);
 				}
 			}
+
+		/* handles stepping correctly when same image is in the list more than once */
+		if (old && old != vw->list_pointer)
+			{
+			gchar *path;
+
+			if (vw->list_pointer)
+				{
+				path = vw->list_pointer->data;
+				}
+			else
+				{
+				path = NULL;
+				}
+
+			image_change_path(imd, path, image_zoom_get_default(imd, zoom_mode));
+			}
 		}
 }
 
