@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -15,9 +15,14 @@
 
 
 void view_window_new(const gchar *path);
+void view_window_new_from_list(GList *list);
 void view_window_new_from_collection(CollectionData *cd, CollectInfo *info);
 
 void view_window_colors_update(void);
 
 
+void view_window_maint_removed(const gchar *path, GList *ignore_list);
+void view_window_maint_moved(const gchar *source, const gchar *dest);
+
+
 #endif
