@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -13,9 +13,11 @@
 #include "gqview.h"
 #include "info.h"
 
+#include "bar_exif.h"
 #include "dnd.h"
 #include "filelist.h"
 #include "image.h"
+#include "ui_bookmark.h"
 #include "ui_fileops.h"
 
 #include <pwd.h>
@@ -25,6 +27,8 @@
 #define IMAGE_SIZE_W 200
 #define IMAGE_SIZE_H 200
 
+#define DEF_PROPERTY_WIDTH  500
+#define DEF_PROPERTY_HEIGHT 300
 
 typedef struct _TabData TabData;
 struct _TabData
@@ -67,6 +71,58 @@ static GtkWidget *table_add_line(GtkWidget *table, gint x, gint y,
 	return label;
 }
 
+/*
+ *-------------------------------------------------------------------
+ * EXIF tab
+ *-------------------------------------------------------------------
+ */
+
+static void info_tab_exif_image(InfoData *id, gpointer data)
+{
+	GtkWidget *bar = data;
+	const gchar *path;
+
+	if (id->image->unknown)
+		{
+		path = NULL;
+		}
+	else
+       		{
+		path = id->image->image_path;
+		}
+
+	bar_exif_set(bar, path);
+}
+
+static void info_tab_exif_sync(InfoData *id, gpointer data)
+{
+	GtkWidget *bar = data;
+
+	bar_exif_set(bar, NULL);
+}
+
+static TabData *info_tab_exif_new(InfoData *id)
+{
+	TabData *td;
+	GtkWidget *bar;
+	GtkWidget *label;
+
+	bar = bar_exif_new(FALSE, NULL, FALSE);
+
+	label = gtk_label_new(_("Exif"));
+	gtk_notebook_append_page(GTK_NOTEBOOK(id->notebook), bar, label);
+	gtk_widget_show(bar);
+
+	/* register */
+	td = g_new0(TabData, 1);
+	td->func_free = NULL;
+	td->func_sync = info_tab_exif_sync;
+	td->func_image = info_tab_exif_image;
+	td->data = bar;
+
+	return td;
+}
+
 /*
  *-------------------------------------------------------------------
  * file attributes tab
@@ -118,11 +174,11 @@ static void info_tab_perm_sync(InfoData *id, gpointer data)
 	InfoTabPerm *tab = data;
 	struct stat st;
 
-	if (stat(id->path, &st) < 0)
+	if (!stat_utf8(id->path, &st))
 		{
-		gtk_label_set(GTK_LABEL(tab->label_user), "");
-		gtk_label_set(GTK_LABEL(tab->label_group), "");
-		gtk_label_set(GTK_LABEL(tab->label_perms), "");
+		gtk_label_set_text(GTK_LABEL(tab->label_user), "");
+		gtk_label_set_text(GTK_LABEL(tab->label_group), "");
+		gtk_label_set_text(GTK_LABEL(tab->label_perms), "");
 		}
 	else
 		{
@@ -133,10 +189,10 @@ static void info_tab_perm_sync(InfoData *id, gpointer data)
 		gchar *buf;
 
 		user = getpwuid(st.st_uid);
-		gtk_label_set(GTK_LABEL(tab->label_user), (user) ? user->pw_name : "");
+		gtk_label_set_text(GTK_LABEL(tab->label_user), (user) ? user->pw_name : "");
 
 		grp = getgrgid(st.st_gid);
-		gtk_label_set(GTK_LABEL(tab->label_group), (grp) ? grp->gr_name : "");
+		gtk_label_set_text(GTK_LABEL(tab->label_group), (grp) ? grp->gr_name : "");
 
 		pbuf[0] = (st.st_mode & S_IRUSR) ? 'r' : ' ';
 		pbuf[1] = (st.st_mode & S_IWUSR) ? 'w' : ' ';
@@ -153,7 +209,7 @@ static void info_tab_perm_sync(InfoData *id, gpointer data)
 
 		pmod = mode_number(st.st_mode);
 		buf = g_strdup_printf("%s (%s)", pbuf, pmod);
-		gtk_label_set(GTK_LABEL(tab->label_perms), buf);
+		gtk_label_set_text(GTK_LABEL(tab->label_perms), buf);
 		g_free(buf);
 		g_free(pmod);
 		}
@@ -205,20 +261,20 @@ struct _InfoTabGeneral
 	GtkWidget *label_image_size;
 	GtkWidget *label_compression;
 	gint compression_done;
-	gint byte_size;
+	gint64 byte_size;
 };
 
 static void info_tab_general_image(InfoData *id, gpointer data)
 {
 	InfoTabGeneral *tab = data;
 	gchar *buf;
-	gint mem_size;
+	guint mem_size;
 	gint has_alpha;
 
 	if (id->image->unknown) return;
 
 	buf = g_strdup_printf("%d x %d", id->image->image_width, id->image->image_height);
-	gtk_label_set(GTK_LABEL(tab->label_dimensions), buf);
+	gtk_label_set_text(GTK_LABEL(tab->label_dimensions), buf);
 	g_free(buf);
 
 	if (id->image->pixbuf)
@@ -229,17 +285,17 @@ static void info_tab_general_image(InfoData *id, gpointer data)
 		{
 		has_alpha = FALSE;
 		}
-	gtk_label_set(GTK_LABEL(tab->label_transparent), has_alpha ? _("yes") : _("no"));
+	gtk_label_set_text(GTK_LABEL(tab->label_transparent), has_alpha ? _("yes") : _("no"));
 
-	mem_size = id->image->image_width * id->image->image_width * 3;
+	mem_size = id->image->image_width * id->image->image_height * ((has_alpha) ? 4 : 3);
 	buf = text_from_size_abrev(mem_size);
-	gtk_label_set(GTK_LABEL(tab->label_image_size), buf);
+	gtk_label_set_text(GTK_LABEL(tab->label_image_size), buf);
 	g_free(buf);
 
 	if (!tab->compression_done && mem_size > 0)
 		{
 		buf = g_strdup_printf("%.1f%%", (float)tab->byte_size / mem_size * 100.0);
-		gtk_label_set(GTK_LABEL(tab->label_compression), buf);
+		gtk_label_set_text(GTK_LABEL(tab->label_compression), buf);
 		g_free(buf);
 
 		tab->compression_done = TRUE;
@@ -251,19 +307,19 @@ static void info_tab_general_sync(InfoData *id, gpointer data)
 	InfoTabGeneral *tab = data;
 	gchar *buf;
 
-	gtk_label_set(GTK_LABEL(tab->label_file_time), text_from_time(filetime(id->path)));
+	gtk_label_set_text(GTK_LABEL(tab->label_file_time), text_from_time(filetime(id->path)));
 
 	tab->byte_size = filesize(id->path);
 
 	buf = text_from_size(tab->byte_size);
-	gtk_label_set(GTK_LABEL(tab->label_file_size), buf);
+	gtk_label_set_text(GTK_LABEL(tab->label_file_size), buf);
 	g_free(buf);
 
-	gtk_label_set(GTK_LABEL(tab->label_dimensions), "");
-	gtk_label_set(GTK_LABEL(tab->label_transparent), "");
-	gtk_label_set(GTK_LABEL(tab->label_image_size), "");
+	gtk_label_set_text(GTK_LABEL(tab->label_dimensions), "");
+	gtk_label_set_text(GTK_LABEL(tab->label_transparent), "");
+	gtk_label_set_text(GTK_LABEL(tab->label_image_size), "");
 
-	gtk_label_set(GTK_LABEL(tab->label_compression), "");
+	gtk_label_set_text(GTK_LABEL(tab->label_compression), "");
 
 	tab->compression_done = FALSE;
 }
@@ -358,6 +414,7 @@ static void info_tabs_init(InfoData *id)
 {
 	id->tab_list = g_list_append(id->tab_list, info_tab_general_new(id));
 	id->tab_list = g_list_append(id->tab_list, info_tab_perm_new(id));
+	id->tab_list = g_list_append(id->tab_list, info_tab_exif_new(id));
 }
 
 /*
@@ -376,15 +433,16 @@ static void info_window_sync(InfoData *id, const gchar *path)
 	if (id->label_count)
 		{
 		gchar *buf;
-		buf = g_strdup_printf("Image %d of %d",
+		buf = g_strdup_printf(_("Image %d of %d"),
 				      g_list_index(id->list, (gpointer)path) + 1,
 				      g_list_length(id->list));
-		gtk_label_set(GTK_LABEL(id->label_count), buf);
+		gtk_label_set_text(GTK_LABEL(id->label_count), buf);
 		g_free(buf);
 		}
 
 	info_tabs_sync(id, FALSE);
 
+	id->updated = FALSE;
 	image_change_path(id->image, path, 0.0);
 }
 
@@ -404,22 +462,28 @@ static void info_window_dnd_data_set(GtkWidget *widget, GdkDragContext *context,
 	path = image_get_path(id->image);
 	if (path)
 		{
-		gchar *text = NULL;
+		gchar *text;
+		gint len;
+		GList *list;
+		gint plain_text;
+
 		switch (info)
 			{
 			case TARGET_URI_LIST:
-				text = g_strconcat("file:", path, "\r\n", NULL);
+				plain_text = FALSE;
 				break;
 			case TARGET_TEXT_PLAIN:
-				text = g_strdup(path);
+			default:
+				plain_text = TRUE;
 				break;
 			}
-		if (text)
-			{
-			gtk_selection_data_set(selection_data, selection_data->target,
-					       8, text, strlen(text));
-			g_free(text);
-			}
+		list = g_list_append(NULL, (gchar *)path);
+		text = uri_text_from_list(list, &len, plain_text);
+		g_list_free(list);
+
+		gtk_selection_data_set(selection_data, selection_data->target,
+				       8, text, len);
+		g_free(text);
 		}
 }
 
@@ -432,16 +496,16 @@ static void info_window_dnd_init(InfoData *id)
 	gtk_drag_source_set(imd->image, GDK_BUTTON2_MASK,
 			    dnd_file_drag_types, dnd_file_drag_types_count,
 			    GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
-	gtk_signal_connect(GTK_OBJECT(imd->image), "drag_data_get",
-			   GTK_SIGNAL_FUNC(info_window_dnd_data_set), id);
+	g_signal_connect(G_OBJECT(imd->image), "drag_data_get",
+			 G_CALLBACK(info_window_dnd_data_set), id);
 
 #if 0
 	gtk_drag_dest_set(imd->image,
 			  GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
 			  dnd_file_drop_types, dnd_file_drop_types_count,
-			  GDK_ACTION_COPY);
-	gtk_signal_connect(GTK_OBJECT(imd->image), "drag_data_received",
-			   GTK_SIGNAL_FUNC(info_window_dnd_data_get), id);
+			  GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
+	g_signal_connect(G_OBJECT(imd->image), "drag_data_received",
+			 G_CALLBACK(info_window_dnd_data_get), id);
 #endif
 }
 
@@ -451,11 +515,24 @@ static void info_window_dnd_init(InfoData *id)
  *-------------------------------------------------------------------
  */
 
+static gint info_window_last_width = DEF_PROPERTY_WIDTH;
+static gint info_window_last_height = DEF_PROPERTY_HEIGHT;
+
 static void info_window_image_update_cb(ImageWindow *imd, gpointer data)
 {
 	InfoData *id = data;
 
+	/* only do this once after when loading a new image,
+	 * for tabs that depend on image data (exif)
+	 * Subsequent updates are ignored, as the image
+	 * should not really changed if id->updated is TRUE.
+	 */
+
+	if (id->updated) return;
+	if (imd->unknown) return;
+
 	info_tabs_sync(id, TRUE);
+	id->updated = TRUE;
 }
 
 static void info_window_back_cb(GtkWidget *widget, gpointer data)
@@ -492,18 +569,39 @@ static void info_window_next_cb(GtkWidget *widget, gpointer data)
 	gtk_widget_set_sensitive(id->button_back, TRUE);
 }
 
+static void info_window_image_button_back(ImageWindow *imd, guint32 time,
+					  gdouble x, gdouble y, guint state, gpointer data)
+{
+	info_window_back_cb(NULL, data);
+}
+
+static void info_window_image_button_next(ImageWindow *imd, guint32 time,
+					  gdouble x, gdouble y, guint state, gpointer data)
+{
+	info_window_next_cb(NULL, data);
+}
+
+static void info_window_close(InfoData *id)
+{
+	gdk_window_get_size(id->window->window, &info_window_last_width, &info_window_last_height);
+	info_window_last_width = MAX(info_window_last_width, DEF_PROPERTY_WIDTH);
+	info_window_last_height = MAX(info_window_last_height, DEF_PROPERTY_HEIGHT);
+
+	gtk_widget_destroy(id->window);
+}
+
 static void info_window_close_cb(GtkWidget *widget, gpointer data)
 {
 	InfoData *id = data;
 
-	gtk_widget_destroy(id->window);
+	info_window_close(id);
 }
 
 static gint info_window_delete_cb(GtkWidget *widget, GdkEventAny *event, gpointer data)
 {
 	InfoData *id = data;
 
-	gtk_widget_destroy(id->window);
+	info_window_close(id);
 	return TRUE;
 }
 
@@ -520,11 +618,12 @@ void info_window_new(const gchar *path, GList *list)
 {
 	InfoData *id;
 	GtkWidget *main_vbox;
-	GtkWidget *main_hbox;
+	GtkWidget *paned;
 	GtkWidget *hbox;
 	GtkWidget *label;
 	GtkWidget *button;
 	GtkWidget *arrow;
+	GdkGeometry geometry;
 
 	if (!path && !list) return;
 
@@ -537,33 +636,49 @@ void info_window_new(const gchar *path, GList *list)
 
 	id->list = list;
 	id->path = (gchar *)id->list->data;
+	id->updated = FALSE;
 
 	id->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	window_set_icon(id->window, NULL, NULL);
 
-	gtk_window_set_policy(GTK_WINDOW(id->window), FALSE, TRUE, FALSE);
+	gtk_window_set_resizable(GTK_WINDOW(id->window), TRUE);
 	gtk_window_set_title(GTK_WINDOW(id->window), _("Image properties - GQview"));
 	gtk_window_set_wmclass(GTK_WINDOW(id->window), "properties", "GQview");
-	gtk_container_border_width(GTK_CONTAINER(id->window), 5);
 
-	gtk_signal_connect(GTK_OBJECT(id->window), "delete_event",
-			   GTK_SIGNAL_FUNC(info_window_delete_cb), id);
-	gtk_signal_connect(GTK_OBJECT(id->window), "destroy",
-			   GTK_SIGNAL_FUNC(info_window_destroy_cb), id);
+	geometry.min_width = 32;
+	geometry.min_height = 32;
+	geometry.base_width = DEF_PROPERTY_WIDTH;
+	geometry.base_height = DEF_PROPERTY_HEIGHT;
+	gtk_window_set_geometry_hints(GTK_WINDOW(id->window), NULL, &geometry,
+				      GDK_HINT_MIN_SIZE | GDK_HINT_BASE_SIZE);
 
-	main_hbox = gtk_hbox_new(FALSE, 5);
-	gtk_container_add(GTK_CONTAINER(id->window), main_hbox);
-	gtk_widget_show(main_hbox);
+
+	gtk_window_set_default_size(GTK_WINDOW(id->window), info_window_last_width, info_window_last_height);
+	gtk_container_set_border_width(GTK_CONTAINER(id->window), 5);
+
+	g_signal_connect(G_OBJECT(id->window), "delete_event",
+			 G_CALLBACK(info_window_delete_cb), id);
+	g_signal_connect(G_OBJECT(id->window), "destroy",
+			 G_CALLBACK(info_window_destroy_cb), id);
+
+	paned = gtk_hpaned_new();
+	gtk_container_add(GTK_CONTAINER(id->window), paned);
+	gtk_widget_show(paned);
 
 	id->image = image_new(FALSE);
 	image_set_update_func(id->image, info_window_image_update_cb, id);
 
-	gtk_widget_set_usize(id->image->widget, IMAGE_SIZE_W, IMAGE_SIZE_H);
-	gtk_box_pack_start(GTK_BOX(main_hbox), id->image->widget, TRUE, TRUE, 0);
+	image_set_button_func(id->image, 1, info_window_image_button_next, id);
+	image_set_button_func(id->image, 5, info_window_image_button_next, id);
+	image_set_button_func(id->image, 2, info_window_image_button_back, id);
+	image_set_button_func(id->image, 4, info_window_image_button_back, id);
+
+	gtk_widget_set_size_request(id->image->widget, IMAGE_SIZE_W, IMAGE_SIZE_H);
+	gtk_paned_pack1(GTK_PANED(paned), id->image->widget, FALSE, TRUE);
 	gtk_widget_show(id->image->widget);
 
 	main_vbox = gtk_vbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(main_hbox), main_vbox, TRUE, TRUE, 0);
+	gtk_paned_pack2(GTK_PANED(paned), main_vbox, TRUE, TRUE);
 	gtk_widget_show(main_vbox);
 
 	hbox = gtk_hbox_new(FALSE, 5);
@@ -574,8 +689,8 @@ void info_window_new(const gchar *path, GList *list)
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	gtk_widget_show(label);
 
-	id->name_entry = gtk_entry_new_with_max_length(512);
-	gtk_widget_set_sensitive(id->name_entry, FALSE);
+	id->name_entry = gtk_entry_new();
+	gtk_editable_set_editable(GTK_EDITABLE(id->name_entry), FALSE);
 	gtk_box_pack_start(GTK_BOX(hbox), id->name_entry, TRUE, TRUE, 0);
 	gtk_widget_show(id->name_entry);
 
@@ -589,23 +704,23 @@ void info_window_new(const gchar *path, GList *list)
 	gtk_widget_show(hbox);
 
 	id->button_back = gtk_button_new();
-	arrow = gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_ETCHED_IN);
+	arrow = gtk_image_new_from_stock(GTK_STOCK_GO_BACK, GTK_ICON_SIZE_BUTTON);
 	gtk_container_add(GTK_CONTAINER(id->button_back), arrow);
 	gtk_widget_show(arrow);
-
-	gtk_signal_connect(GTK_OBJECT(id->button_back), "clicked",
-			   GTK_SIGNAL_FUNC(info_window_back_cb), id);
+	
+	g_signal_connect(G_OBJECT(id->button_back), "clicked",
+			 G_CALLBACK(info_window_back_cb), id);
 	gtk_widget_set_sensitive(id->button_back, FALSE);
 	gtk_box_pack_start(GTK_BOX(hbox), id->button_back, FALSE, FALSE, 0);
 	gtk_widget_show(id->button_back);
 
 	id->button_next = gtk_button_new();
-	arrow = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_ETCHED_IN);
+	arrow = gtk_image_new_from_stock(GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_BUTTON);
 	gtk_container_add(GTK_CONTAINER(id->button_next), arrow);
 	gtk_widget_show(arrow);
 
-	gtk_signal_connect(GTK_OBJECT(id->button_next), "clicked",
-			   GTK_SIGNAL_FUNC(info_window_next_cb), id);
+	g_signal_connect(G_OBJECT(id->button_next), "clicked",
+			 G_CALLBACK(info_window_next_cb), id);
 	gtk_widget_set_sensitive(id->button_next, (id->list->next != NULL));
 	gtk_box_pack_start(GTK_BOX(hbox), id->button_next, FALSE, FALSE, 0);
 	gtk_widget_show(id->button_next);
@@ -617,9 +732,9 @@ void info_window_new(const gchar *path, GList *list)
 		gtk_widget_show(id->label_count);
 		}
 
-	button = gtk_button_new_with_label(_("Close"));
-	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			   GTK_SIGNAL_FUNC(info_window_close_cb), id);
+	button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
+	g_signal_connect(G_OBJECT(button), "clicked",
+			 G_CALLBACK(info_window_close_cb), id);
 	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
