@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -13,6 +13,7 @@
 #include "gqview.h"
 #include "rcfile.h"
 
+#include "filelist.h"
 #include "ui_fileops.h"
 
 
@@ -22,9 +23,9 @@
  *-----------------------------------------------------------------------------
  */ 
 
-gchar *quoted_value(gchar *text)
+gchar *quoted_value(const gchar *text)
 {
-	gchar *ptr;
+	const gchar *ptr;
 	gint c = 0;
 	gint l = strlen(text);
 
@@ -33,15 +34,16 @@ gchar *quoted_value(gchar *text)
 	while (c < l && text[c] !='"') c++;
 	if (text[c] == '"')
 		{
+		gint e;
 		c++;
 		ptr = text + c;
-		while (c < l && text[c] !='"') c++;
-		if (text[c] == '"')
+		e = c;
+		while (e < l && text[e] !='"') e++;
+		if (text[e] == '"')
 			{
-			text[c] = '\0';
-			if (strlen(ptr) > 0)
+			if (e - c > 0)
 				{
-				return g_strdup(ptr);
+				return g_strndup(ptr, e - c);
 				}
 			}
 		}
@@ -53,8 +55,7 @@ gchar *quoted_value(gchar *text)
 		while (c < l && text[c] !=' ' && text[c] !=8 && text[c] != '\n') c++;
 		if (c != 0)
 			{
-			text[c] = '\0';
-			return g_strdup(text);
+			return g_strndup(text, c);
 			}
 		}
 
@@ -71,7 +72,7 @@ static void write_char_option(FILE *f, gchar *label, gchar *text)
 
 static gchar *read_char_option(FILE *f, gchar *option, gchar *label, gchar *value, gchar *text)
 {
-	if (!strcasecmp(option, label))
+	if (strcasecmp(option, label) == 0)
 		{
 		g_free(text);
 		text = quoted_value(value);
@@ -86,7 +87,7 @@ static void write_int_option(FILE *f, gchar *label, gint n)
 
 static gint read_int_option(FILE *f, gchar *option, gchar *label, gchar *value, gint n)
 {
-	if (!strcasecmp(option, label))
+	if (strcasecmp(option, label) == 0)
 		{
 		n = strtol(value,NULL,0);
 		}
@@ -102,9 +103,9 @@ static void write_bool_option(FILE *f, gchar *label, gint n)
 
 static gint read_bool_option(FILE *f, gchar *option, gchar *label, gchar *value, gint n)
 {
-	if (!strcasecmp(option, label))
+	if (strcasecmp(option, label) == 0)
 		{
-		if (!strcasecmp(value, "true"))
+		if (strcasecmp(value, "true") == 0)
 			n = TRUE;
 		else
 			n = FALSE;
@@ -122,14 +123,22 @@ void save_options(void)
 {
 	FILE *f;
 	gchar *rc_path;
+	gchar *rc_pathl;
 	gint i;
 
 	rc_path = g_strconcat(homedir(), "/", GQVIEW_RC_DIR, "/", RC_FILE_NAME, NULL);
 
-	f = fopen(rc_path,"w");
+	rc_pathl = path_from_utf8(rc_path);
+	f = fopen(rc_pathl, "w");
+	g_free(rc_pathl);
 	if (!f)
 		{
-		printf(_("error saving config file: %s\n"), rc_path);
+		gchar *buf;
+
+		buf = g_strdup_printf(_("error saving config file: %s\n"), rc_path);
+		print_term(buf);
+		g_free(buf);
+
 		g_free(rc_path);
 		return;
 		}
@@ -149,10 +158,13 @@ void save_options(void)
 	fprintf(f,"\n");
 	write_bool_option(f, "layout_view_as_icons", layout_view_icons);
 	write_bool_option(f, "layout_view_as_tree", layout_view_tree);
+	write_bool_option(f, "show_icon_names", show_icon_names);
 	fprintf(f,"\n");
 
 	write_bool_option(f, "tree_descend_folders", tree_descend_subdirs);
 	write_bool_option(f, "lazy_image_sync", lazy_image_sync);
+	write_bool_option(f, "update_on_time_change", update_on_time_change);
+	write_bool_option(f, "exif_auto_rotate", exif_rotate_enable);
 	fprintf(f,"\n");
 
 	write_bool_option(f, "enable_startup_path", startup_path_enable);
@@ -187,10 +199,14 @@ void save_options(void)
 
 	write_int_option(f, "sort_method", (gint)file_sort_method);
 	write_bool_option(f, "sort_ascending", file_sort_ascending);
+	write_bool_option(f, "sort_case_sensitive", file_sort_case_sensitive);
 	fprintf(f,"\n");
 
 	write_bool_option(f, "confirm_delete", confirm_delete);
 	write_bool_option(f, "enable_delete_key", enable_delete_key);
+	write_bool_option(f, "safe_delete", safe_delete_enable);
+	write_char_option(f, "safe_delete_path", safe_delete_path);
+	write_int_option(f, "safe_delete_size", safe_delete_size);
 	fprintf(f,"\n");
 	write_bool_option(f, "tools_float", tools_float);
 	write_bool_option(f, "tools_hidden", tools_hidden);
@@ -213,7 +229,7 @@ void save_options(void)
 
 	write_bool_option(f, "enable_read_ahead", enable_read_ahead);
 
-	write_bool_option(f, "place_dialogs_under_mouse", place_dialogs_under_mouse);
+	write_bool_option(f, "display_dialogs_under_mouse", place_dialogs_under_mouse);
 
 	write_bool_option(f, "black_window_background", black_window_background);
 
@@ -230,24 +246,13 @@ void save_options(void)
 
 	write_bool_option(f, "show_dotfiles", show_dot_files);
 	write_bool_option(f, "disable_filtering", file_filter_disable);
-	fprintf(f,"\n");
-	write_bool_option(f, "filter_ppm", filter_include_ppm);
-	write_bool_option(f, "filter_png", filter_include_png);
-	write_bool_option(f, "filter_jpg", filter_include_jpg);
-	write_bool_option(f, "filter_tif", filter_include_tif);
-	write_bool_option(f, "filter_pgm", filter_include_pgm);
-	write_bool_option(f, "filter_xpm", filter_include_xpm);
-	write_bool_option(f, "filter_gif", filter_include_gif);
-	write_bool_option(f, "filter_pcx", filter_include_pcx);
-	write_bool_option(f, "filter_bmp", filter_include_bmp);
-	fprintf(f,"\n");
-	write_char_option(f, "custom_filter", custom_filter);
+	filter_write_list(f);
 
 	fprintf(f,"\n##### External Programs #####\n");
 	fprintf(f,"# Maximum of 8 programs (external_1 through external 8)\n");
 	fprintf(f,"# format: external_n: \"menu name\" \"command line\"\n\n");
 
-	for (i=0; i<8; i++)
+	for (i = 0; i < GQVIEW_EDITOR_SLOTS; i++)
 		{
 		fprintf(f,"external_%d: \"", i+1);
 		if (editor_name[i]) fprintf(f, "%s", editor_name[i]);
@@ -259,7 +264,6 @@ void save_options(void)
 	fprintf(f,"\n##### Collection Options #####\n\n");
 
 	write_bool_option(f, "rectangular_selections", collection_rectangular_selection);
-	write_bool_option(f, "collection_drop_insert", collection_drop_insert);
 
 	fprintf(f,"\n##### Window Positions #####\n\n");
 
@@ -296,6 +300,7 @@ void load_options(void)
 {
 	FILE *f;
 	gchar *rc_path;
+	gchar *rc_pathl;
 	gchar s_buf[1024];
 	gchar *s_buf_ptr;
 	gchar option[1024];
@@ -305,7 +310,9 @@ void load_options(void)
 
 	rc_path = g_strconcat(homedir(), "/", GQVIEW_RC_DIR, "/", RC_FILE_NAME, NULL);
 
-	f = fopen(rc_path,"r");
+	rc_pathl = path_from_utf8(rc_path);
+	f = fopen(rc_pathl,"r");
+	g_free(rc_pathl);
 	if (!f)
 		{
 		g_free(rc_path);
@@ -322,9 +329,7 @@ void load_options(void)
 		if (c >= l) continue;
 		s_buf[c] = '\0';
 		c++;
-		while (s_buf[c] == ' ' && c < l) c++;
-		while (s_buf[c] == 8 && c < l) c++;
-		while (s_buf[c] == ' ' && c < l) c++;
+		while ((s_buf[c] == ' ' || s_buf[c] == 8) && c < l) c++;
 		s_buf_ptr = s_buf + c;
 		strncpy(value_all, s_buf_ptr, sizeof(value_all));
 		while (s_buf[c] != 8 && s_buf[c] != ' ' && s_buf[c] != '\n' && c < l) c++;
@@ -342,22 +347,28 @@ void load_options(void)
 			"layout_view_as_icons", value, layout_view_icons);
 		layout_view_tree = read_bool_option(f, option,
 			"layout_view_as_tree", value, layout_view_tree);
+		show_icon_names = read_bool_option(f, option,
+			"show_icon_names", value, show_icon_names);
 
 		tree_descend_subdirs = read_bool_option(f, option,
 			"tree_descend_folders", value, tree_descend_subdirs);
 		lazy_image_sync = read_bool_option(f, option,
 			"lazy_image_sync", value, lazy_image_sync);
+		update_on_time_change = read_bool_option(f, option,
+			"update_on_time_change", value, update_on_time_change);
+		exif_rotate_enable = read_bool_option(f, option,
+			"exif_auto_rotate", value, exif_rotate_enable);
 
 		startup_path_enable = read_bool_option(f, option,
 			"enable_startup_path", value, startup_path_enable);
 		startup_path = read_char_option(f, option,
 			"startup_path", value_all, startup_path);
 
-		if (!strcasecmp(option,"zoom_mode"))
+		if (strcasecmp(option,"zoom_mode") == 0)
                         {
-                        if (!strcasecmp(value,"original")) zoom_mode = ZOOM_RESET_ORIGINAL;
-                        if (!strcasecmp(value,"fit")) zoom_mode = ZOOM_RESET_FIT_WINDOW;
-                        if (!strcasecmp(value,"dont_change")) zoom_mode = ZOOM_RESET_NONE;
+                        if (strcasecmp(value, "original") == 0) zoom_mode = ZOOM_RESET_ORIGINAL;
+                        if (strcasecmp(value, "fit") == 0) zoom_mode = ZOOM_RESET_FIT_WINDOW;
+                        if (strcasecmp(value, "dont_change") == 0) zoom_mode = ZOOM_RESET_NONE;
                         }
 		two_pass_zoom = read_bool_option(f, option,
 			"two_pass_scaling", value, two_pass_zoom);
@@ -390,11 +401,19 @@ void load_options(void)
 			"sort_method", value, (gint)file_sort_method);
 		file_sort_ascending = read_bool_option(f, option,
 			"sort_ascending", value, file_sort_ascending);
+		file_sort_case_sensitive = read_bool_option(f, option,
+			"sort_case_sensitive", value, file_sort_case_sensitive);
 
 		confirm_delete = read_bool_option(f, option,
 			"confirm_delete", value, confirm_delete);
 		enable_delete_key = read_bool_option(f, option,
 			"enable_delete_key", value, enable_delete_key);
+		safe_delete_enable = read_bool_option(f, option,
+			"safe_delete",  value, safe_delete_enable);
+		safe_delete_path = read_char_option(f, option,
+			"safe_delete_path", value, safe_delete_path);
+		safe_delete_size = read_int_option(f, option,
+			"safe_delete_size", value, safe_delete_size);
 
 		tools_float = read_bool_option(f, option,
 			"tools_float", value, tools_float);
@@ -431,7 +450,7 @@ void load_options(void)
 			"enable_read_ahead", value, enable_read_ahead);
 
 		place_dialogs_under_mouse = read_bool_option(f, option,
-			"place_dialogs_under_mouse", value, place_dialogs_under_mouse);
+			"display_dialogs_under_mouse", value, place_dialogs_under_mouse);
 
 		black_window_background = read_bool_option(f, option,
 			"black_window_background", value, black_window_background);
@@ -455,34 +474,17 @@ void load_options(void)
 		file_filter_disable = read_bool_option(f, option,
 			"disable_filtering", value, file_filter_disable);
 
-		filter_include_ppm = read_bool_option(f, option,
-			"filter_ppm", value, filter_include_ppm);
-		filter_include_png = read_bool_option(f, option,
-			"filter_png", value, filter_include_png);
-		filter_include_jpg = read_bool_option(f, option,
-			"filter_jpg", value, filter_include_jpg);
-		filter_include_tif = read_bool_option(f, option,
-			"filter_tif", value, filter_include_tif);
-		filter_include_pgm = read_bool_option(f, option,
-			"filter_pgm", value, filter_include_pgm);
-		filter_include_xpm = read_bool_option(f, option,
-			"filter_xpm", value, filter_include_xpm);
-		filter_include_gif = read_bool_option(f, option,
-			"filter_gif", value, filter_include_gif);
-		filter_include_pcx = read_bool_option(f, option,
-			"filter_pcx", value, filter_include_pcx);
-		filter_include_bmp = read_bool_option(f, option,
-			"filter_bmp", value, filter_include_bmp);
-
-		custom_filter = read_char_option(f, option,
-			"custom_filter", value_all, custom_filter);
+		if (strcasecmp(option, "filter_ext") == 0)
+			{
+			filter_parse(value_all);
+			}
 
 		/* External Programs */
 
-		if (!strncasecmp(option,"external_",9))
+		if (strncasecmp(option, "external_", 9) == 0)
 			{
 			i = strtol(option + 9, NULL, 0);
-			if (i>0 && i<9)
+			if (i > 0 && i <= GQVIEW_EDITOR_SLOTS)
 				{
 				gchar *ptr1, *ptr2;
 				i--;
@@ -512,8 +514,9 @@ void load_options(void)
 							{
 							c++;
 							ptr2 = ptr1 + c;
-							while (c<l && value_all[c] !='"') c++;
-							if (ptr1[c] == '"' && ptr1 + c - 1 != ptr2)
+							while (value_all[c] != '\0') c++;
+							while (c > 0 && value_all[c] != '"') c--;
+							if (ptr1[c] == '"' && ptr1 + c > ptr2)
 								{
 								ptr1[c] = '\0';
 								editor_command[i] = g_strdup(ptr2);
@@ -528,8 +531,6 @@ void load_options(void)
 
 		collection_rectangular_selection = read_bool_option(f, option,
 			"rectangular_selections", value, collection_rectangular_selection);
-		collection_drop_insert = read_bool_option(f, option,
-			"collection_drop_insert", value, collection_drop_insert);
 
 		/* window positions */
 
